"""Gateway connecting to ATAG thermostat."""
import logging
import datetime
import asyncio
from aiohttp import ClientSession
from .helpers import HostConfig, HttpConnector, get_data_from_jsonreply, check_reply
from .errors import ResponseError, RequestError
from .const import BOILER_STATUS, MODES

_LOGGER = logging.getLogger(__name__)
MINTIMEBETWEENCALLS = 5  # Time in seconds


class AtagDataStore:
    """Central data store entity."""

    def __init__(
        self,
        session=None,
        host=None,
        port=10000,
        device=None,
        hostname="HomeAssistant",
        mail=None,
        auth=None,
        ssl=None,
        proxy=None,
        paired=False,
        **kwargs
    ):

        self._initialized = False
        self._device = device
        self._paired = paired
        self.connection = None
        self.sensordata = {}
        self._session = session
        mail = mail or kwargs.get("email")
        self.config = HostConfig(host, port, mail, hostname, ssl, proxy)
        self._last_api_call = (
            datetime.datetime(1970, 1, 1, 0, 0, 0, tzinfo=datetime.timezone.utc),
            None,
        )
        self._target_temperature = None
        self._dhw_target_temperature = None
        self._hvac_mode = None
        self._hold_mode = None
        if self.config.host:
            self.initialized = True

    @property
    def device(self):
        """Device identifier"""
        return self._device

    @property
    def apiversion(self):
        """Return api version reported by device"""
        return self.sensordata.get("download_url").get("state").split("/")[-1]

    @property
    def paired(self):
        """Pairing status"""
        return self._paired

    @property
    def initialized(self):
        """Initialization completed"""
        return self._initialized

    @initialized.setter
    def initialized(self, arg=None):
        if not self.config.host:
            raise RequestError("Attempted initialization without hostaddress")
        if self._session is None:
            self._session = ClientSession()
        self.connection = HttpConnector(self.config, self._session)
        self._initialized = True

    @property
    def burner_status(self):
        """Returns a tuple: boolean for burning and percentage for modulation"""
        if isinstance(self.sensordata.get(BOILER_STATUS).get("state"), list):
            if self.sensordata[BOILER_STATUS]["state"][0] == 1:
                return (True, self.sensordata.get("rel_mod_level"))
        return (False, 0)

    @property
    def cv_status(self):
        """Return boolean indicator for heating for CV"""
        if self.burner_status:
            return self.sensordata[BOILER_STATUS]["state"][2] == 1

    @property
    def dhw_status(self):
        """Return boolean indicator for heating for DHW"""
        if self.burner_status:
            return self.sensordata[BOILER_STATUS]["state"][1] == 1

    @property
    def hvac_mode(self):
        """Return the operating mode (Weather or Regular/Heat)"""
        if (
            self._last_api_call[1] == "ch_control_mode"
            and self.sensordata.get("report_time").get("state") < self._last_api_call[0]
        ):
            return self._hvac_mode
        return self.sensordata.get("ch_control_mode").get("state")

    async def set_hvac_mode(self, mode: str) -> bool:
        """Set the operating mode (Weather or Regular/Heat) """
        if mode == self.hvac_mode:
            return True
        int_mode = 1 if mode == "auto" else 0
        try:
            if await self.async_set_atag(ch_control_mode=int_mode):
                self._hvac_mode = mode
                return True
        except ResponseError as err:
            _LOGGER.error("Failed to update mode: %s", err)
            return False

    @property
    def hold_mode(self):
        """Return the hold mode (Manual/Auto/Extend/Vacation/Fireplace)"""
        if (
            self._last_api_call[1] == "ch_mode"
            and self.sensordata.get("report_time").get("state") < self._last_api_call[0]
        ):
            return self._hold_mode
        return self.sensordata.get("ch_mode").get("state")

    @property
    def hold_mode_duration(self):
        """Return remaining time on hold mode"""
        if self.hold_mode == "extend":
            seconds = self.sensordata.get("ch_mode_duration").get("state")
            return datetime.timedelta(seconds=seconds)

    async def set_hold_mode(self, mode: str, **kwargs) -> bool:
        """Set the hold mode (Manual/Auto/Extend/Vacation/Fireplace) """
        if mode == self.hold_mode:
            return True
        int_mode = MODES.get(mode)
        if not int_mode:
            _LOGGER.error("Received invalid mode: %s", mode)
            return False
        try:
            if await self.async_set_atag(ch_mode=int_mode):
                self._hold_mode = mode
                return True
        except ResponseError as err:
            _LOGGER.error("Failed to update mode: %s", err)
            return False

    @property
    def temperature(self):
        """Return current CV temperature"""
        return self.sensordata.get("room_temp").get("state")

    @property
    def target_temperature(self):
        """Return target CV temperature"""
        if (
            self._last_api_call[1] == "temperature"
            and self.sensordata.get("report_time").get("state") < self._last_api_call[0]
        ):
            return self._target_temperature
        return self.sensordata.get("ch_mode_temp").get("state")

    async def set_temp(self, target: float):
        """Set target CV temperature"""
        if target == self.target_temperature:
            return True
        try:
            if await self.async_set_atag(temperature=target):
                self._target_temperature = target
                return True
        except ResponseError as err:
            _LOGGER.error("Failed to set atag: %s", err)
            return False

    @property
    def dhw_temperature(self):
        """Return current dhw temperature"""
        return self.sensordata.get("dhw_water_temp").get("state")

    @property
    def dhw_min_temp(self):
        """Return dhw min temperature"""
        return self.sensordata.get("dhw_min_set").get("state")

    @property
    def dhw_max_temp(self):
        """Return dhw max temperature"""
        return self.sensordata.get("dhw_max_set").get("state")

    @property
    def dhw_target_temperature(self):
        """Return dhw target temperature"""

        if self.dhw_status:
            return self.sensordata.get("dhw_temp_setp").get("state")

        if (
            self._last_api_call[1] == "dhw_mode_temp"
            and self.sensordata.get("report_time").get("state") < self._last_api_call[0]
        ):
            return self._dhw_target_temperature
        return self.sensordata.get("dhw_mode_temp").get("state") % 150

    async def dhw_set_temp(self, target: float):
        """Set dhw target temperature"""
        if target == self.dhw_target_temperature:
            return True
        try:
            if await self.async_set_atag(dhw_target_temp=target):
                self._dhw_target_temperature = target
                return True
        except ResponseError as err:
            _LOGGER.error("Failed to set atag: %s", err)
            return False

    async def async_host_search(self):
        """Atag Discovery in case no host provided."""
        from .discovery import discover_atag

        _LOGGER.debug("No host provided, attempting discovery...")
        host, device = await discover_atag()
        if host and device:
            self.config.host = host
            self._device = device
            self.config.update_params()

    async def async_update(self):
        """Read data from thermostat."""
        if not self.initialized:
            self.initialized = True

        if not self.paired:
            await self.async_check_pair_status()
        await self.can_call()
        try:
            json_data = await self.connection.atag_put(data=self.config.retrieve_msg)
            sensordata = get_data_from_jsonreply(json_data)
            if sensordata:
                self.sensordata = sensordata
        except ResponseError as err:
            _LOGGER.warning("Failed to update Atag Data: %s", err)

    async def async_set_atag(self, **kwargs):
        """set mode and/or temperature."""

        payload = self.config.get_update_msg(**kwargs)
        await self.can_call()
        try:
            self._last_api_call = (
                datetime.datetime.now().astimezone(),
                list(kwargs)[0],
            )
            json_data = await self.connection.atag_put(payload)
            _LOGGER.debug("Update reply: %s", json_data)
            return check_reply(json_data) == 2
        except (ResponseError, KeyError) as err:
            raise RequestError("Failed to set Atag: {}".format(err))

    async def async_check_pair_status(self):
        """Confirm we are authorized."""

        if self.paired:
            return True
        if not self.initialized:
            self.initialized = True
        await self.can_call()
        try:
            self._last_api_call = (datetime.datetime.now().astimezone(), None)
            json_data = await self.connection.atag_put(self.config.pair_msg)
            status = check_reply(json_data)
            if status == 2 and self.device is not None:
                self._paired = True
            elif status == 2:
                json_data = await self.connection.atag_put(
                    self.config.get_retrieve_msg(info=0)
                )
                self._device = json_data["retrieve_reply"]["status"]["device_id"]
                self._paired = True
            else:
                self._paired = False
                raise ResponseError("Pairing failed")
        except ResponseError as err:
            self._paired = False
            raise RequestError("Pairing failed: {}".format(err))

    async def async_close(self):
        """Close the connection"""
        await self.connection.async_close()

    async def can_call(self):
        """Sleep until next API call can be made, to avoid device overload"""
        slept = min(
            datetime.datetime.now().astimezone() - self._last_api_call[0],
            datetime.timedelta(seconds=MINTIMEBETWEENCALLS),
        ).seconds
        await asyncio.sleep(MINTIMEBETWEENCALLS - slept)

        return True



"""Helpers for the ATAG connection"""

from datetime import datetime, timezone, timedelta
from numbers import Number
import logging
import json
import re
import uuid

import asyncio
from aiohttp import client_exceptions
from .errors import RequestError, ResponseError
from .const import (
    REQUEST_INFO,
    SENSOR_VALUES,
    BOILER_STATUS,
    DEFAULT_TIMEOUT,
    DETAILS,
    REPORT,
    CONTROLS,
    HTTP_HEADER,
)

MAC = "mac"
HOSTNAME = "hostname"
MAIL = "mail"
URL = "url"
HOST = "host"
PORT = "port"
DEVICE = "device"

RETRIEVE_REPLY = "retrieve_reply"

LOCALTZ = datetime.now().astimezone().tzinfo
_LOGGER = logging.getLogger(__name__)


def get_data_from_jsonreply(json_response):
    """Return relevant sensor data from json retrieve reply."""
    result = {}
    try:
        _reply = json_response[RETRIEVE_REPLY]
        _reply[DETAILS] = _reply[REPORT].pop(DETAILS)
        for group in _reply.keys():
            if group in ["seqnr", "acc_status"]:
                continue
            for key in _reply[group]:
                if key in SENSOR_VALUES:
                    worker = int(_reply[group][key])
                    result[key] = get_state_from_worker(key, worker)
                else:
                    res = _reply[group][key]
                    if isinstance(res, Number):
                        result[key] = {'state': float(res)}
                    else:
                        result[key] = {'state': res}
    except KeyError as err:
        raise ResponseError("Invalid value {} for {}".format(err, group))
    return result


def check_reply(json_reply):
    """reutrn the account status in an atag reply."""
    return json_reply[list(json_reply.keys())[0]]["acc_status"]


def get_state_from_worker(key, worker):
    """
    Returns:\n
    Sensor values when decoded (int based).\n
    Binary representation when not decoded.\n
    Time based on seconds from 2000 (UTC).
    """
    if key == BOILER_STATUS:
        return {'state': list(map(int, list("{0:04b}".format(worker & 14))))[0:3]}  
    if SENSOR_VALUES[key] == "time":
        return {'state': (
            datetime(2000, 1, 1, tzinfo=timezone.utc) +
            timedelta(seconds=worker)
        ).astimezone(tz=LOCALTZ)}
    if SENSOR_VALUES[key] == "int":
        # TODO not yet decoded integer values
        return {'state_orig': worker, 'state': int_to_binary(worker)}
    if worker in SENSOR_VALUES[key]:
        return SENSOR_VALUES[key][worker]
    return {'state': worker}  # not yet figured out what it means


def int_to_binary(worker):
    """Returns binary representation of int (for certain status/config values)."""
    return "{0:b}".format(worker)


class HttpConnector:
    """HTTP connector to ATAG thermostat."""

    def __init__(self, hostconfig, session=None):
        """Init of HTTP connector."""
        self.hostconfig = hostconfig
        self._websession = session
        self._request_timeout = DEFAULT_TIMEOUT

    async def atag_put(self, data):
        """Make a put request to the API."""
        path = list(data)[0].split("_")[0]
        posturl = "{}{}".format(self.hostconfig.baseurl, path)
        _LOGGER.debug("Posting to %s", posturl)
        try:
            async with self._websession.put(
                posturl,
                json=data,
                headers=self.hostconfig.http_header,
                proxy=self.hostconfig.proxy,
                # ssl=self.hostconfig.ssl,
                timeout=self._request_timeout,
            ) as req:
                data = await req.text()
                _LOGGER.debug("Received: %s", data)
                json_result = json.loads(data)
                return json_result
        except (
            client_exceptions.ClientError,
            ConnectionResetError,
            json.JSONDecodeError,
            client_exceptions.ClientConnectorError,
            asyncio.TimeoutError,
        ) as err:
            raise ResponseError(err)

    def set_timeout(self, timeout=DEFAULT_TIMEOUT):
        """Set timeout for API calls."""
        self._request_timeout = timeout

    async def async_close(self):
        """Close the connection"""
        await self._websession.close()


class HostConfig:
    """Connection info store."""

    def __init__(
        self,
        host=None,
        port=10000,
        mail=None,
        hostname="HomeAssistant",
        ssl=None,
        proxy=None,
    ):

        self.host = host
        self.port = port or 10000
        self.mail = mail or ""
        self.proxy = proxy
        self._mac = ":".join(re.findall("..", "%012x" % uuid.getnode()))
        self._ssl = ssl
        self.http_header = HTTP_HEADER
        self.hostname = hostname
        self.set_pair_msg()
        self.retrieve_msg = self.get_retrieve_msg()
        if self.host:
            self.update_params()
        # self.http_header['Authorization'] = host_config.get('auth')

    def update_params(self, host=None):
        """Set or update the values that depend on host address"""
        if host:
            self.host = host
        self.baseurl = "{}:{}/".format(self.host, self.port)
        if not re.match(r"^https?:\/\/", self.baseurl):
            prefix = "https://" if self._ssl else "http://"
            self.baseurl = prefix + self.baseurl

    def get_retrieve_msg(self, info=REQUEST_INFO):
        """Get and store the constant retrieve payload."""

        json_payload = {
            "retrieve_message": {
                "seqnr": 1,
                "account_auth": {"user_account": self.mail, "mac_address": self._mac},
                "info": info,
            }
        }
        return json_payload

    def set_pair_msg(self, hass=True):
        """Get and store the constant pairing payload."""

        msg = {
            "pair_message": {
                "seqnr": 1,
                "account_auth": {"user_account": self.mail, "mac_address": self._mac},
                "accounts": {
                    "entries": [
                        {
                            "user_account": self.mail,
                            "mac_address": self._mac,
                            "device_name": self.hostname,
                            "account_type": 1,
                        }
                    ]
                },
            }
        }
        self.pair_msg = msg

    def get_update_msg(self, **kwargs):
        """Return the update payload for control input."""
        for key, val in kwargs.items():
            if not key in CONTROLS or not isinstance(val, Number):
                raise RequestError("Invalid value for {}: {}".format(key, val))

        json_payload = {
            "update_message": {
                "seqnr": 1,
                "account_auth": {"user_account": self.mail, "mac_address": self._mac},
                "control": {},
            }
        }

        for key, val in kwargs.items():
            json_payload["update_message"]["control"][CONTROLS[key]] = val

        return json_payload



"""Constants for ATAG API."""

import datetime

STATE_HEAT = "heat"
STATE_ECO = "eco"
STATE_AUTO = "Auto"
STATE_MANUAL = "Manual"
STATE_EXTEND = "Extend"
STATE_HOLIDAY = "holiday"
# STATE_FIREPLACE = 'fireplace'
STATE_FIREPLACE = "boost"
STATE_OFF = "off"

DEFAULT_TIMEOUT = 15
DEFAULT_PORT = 10000
DEFAULT_SCAN_INTERVAL = 30
DEFAULT_MIN_TEMP = 12
DEFAULT_MAX_TEMP = 21
REQUEST_INFO = 71

HTTP_HEADER = {
    "Content-type": "applicaton/json;charset=UTF-8",
    "Connection": "keep-alive",
    "User-Agent": "Mozilla/5.0 (compatible; AtagOneAPI/x; http://atag.one/)",
}

# CH constants
CH_STATE = "ch_status"
CH_CURRENT_TEMPERATURE = "current_temperature"
CH_MAX_TEMP = "max_temp"
CH_MIN_TEMP = "min_temp"
CH_OPERATION_LIST = "operation_list"
CH_OPERATION_MODE = "operation_mode"
CH_MODE = "ch_mode"  # same as CH_OPERATION_MODE
CH_TEMPERATURE = "temperature"
CH_OPERATION_CONTROL = "ch_control_mode"  # TBD HA
CH_MODE_DURATION = "ch_mode_duration"  # TBD HA
CH_HOLD_DURATION = "extend_duration"  # TBD HA
VACATION = "vacation_duration"  # TBD HA
FIREPLACE_DURATION = "fireplace_duration"  # TBD HA
REPORT_TIME = "report_time"
BOILER_STATUS = "boiler_status"
BOILER_CONF = "boiler_config"

# DHW constants
DHW_STATE = "dhw_status"
DHW_CURRENT_TEMPERATURE = "dhw_water_temp"
DHW_TEMPERATURE = "dhw_temp_setp"  # TBD HA
DHW_OPERATION_MODE = "dhw_mode"  # TBD HA
DHW_MODE_TEMPERATURE = "dhw_mode_temp"  # TBD HA

STATUS = "status"
REPORT = "report"
CONTROL = "control"
DETAILS = "details"
ACC_STATUS = "acc_status"
WEATHER_STATUS = "weather_status"
WEATHER_CURRENT_TEMPERATURE = "weather_current_temperature"

CONTROLS = {
    CH_STATE: "ch_status",
    CH_OPERATION_CONTROL: "ch_control_mode",
    CH_OPERATION_MODE: "ch_mode",
    CH_MODE: "ch_mode",
    CH_MODE_DURATION: "ch_mode_duration",
    CH_TEMPERATURE: "ch_mode_temp",
    CH_HOLD_DURATION: "extend_duration",
    FIREPLACE_DURATION: "fireplace_duration",
    VACATION: "vacation_duration",
    DHW_TEMPERATURE: "dhw_temp_setp",
    DHW_STATE: "dhw_status",
    DHW_OPERATION_MODE: "dhw_mode",
    DHW_MODE_TEMPERATURE: "dhw_mode_temp",
    WEATHER_STATUS: "weather_status",
    WEATHER_CURRENT_TEMPERATURE: "weather_temp",
}

REPORT_STRUCTURE = {
    STATUS: ["device_id", "device_status", "connection_status", "date_time"],
    REPORT: [
        "report_time",
        "burning_hours",
        "device_errors",
        "boiler_errors",
        "room_temp",
        "outside_temp",
        "dbg_outside_temp",
        "pcb_temp",
        "ch_setpoint",
        "dhw_water_temp",
        "ch_water_temp",
        "dhw_water_pres",
        "ch_water_pres",
        "ch_return_temp",
        "boiler_status",
        "boiler_config",
        "ch_time_to_temp",
        "shown_set_temp",
        "power_cons",
        "tout_avg",
        "rssi",
        "current",
        "voltage",
        "charge_status",
        "lmuc_burner_starts",
        "dhw_flow_rate",
        "resets",
        "memory_allocation",
    ],
    DETAILS: [
        "boiler_temp",
        "boiler_return_temp",
        "min_mod_level",
        "rel_mod_level",
        "boiler_capacity",
        "target_temp",
        "overshoot",
        "max_boiler_temp",
        "alpha_used",
        "regulation_state",
        "ch_m_dot_c",
        "c_house",
        "r_rad",
        "r_env",
        "alpha",
        "alpha_max",
        "delay",
        "mu",
        "threshold_offs",
        "wd_k_factor ",
        "wd_exponent",
        "lmuc_burner_hours",
        "lmuc_dhw_hours",
        "KP",
        "KI",
    ],
    CONTROL: [
        "ch_status",
        "ch_control_mode",
        "ch_mode",
        "ch_mode_duration",
        "ch_mode_temp",
        "dhw_temp_setp",
        "dhw_status",
        "dhw_mode",
        "dhw_mode_temp",
        "weather_temp",
        "weather_status",
        "vacation_duration",
        "extend_duration",
        "fireplace_duration",
    ],
}
SENSOR_TYPES = {
    "device_id": {
        "type": "One ID",
        "unit": "",
        "icon": "mdi:account-card-details-outline",
        "datafield": "device_id",
    },
    "device_status": {
        "type": "One status",
        "unit": "",
        "icon": "mdi:account-card-details-outline",
        "datafield": "device_status",
    },
    "connection_status": {
        "type": "Connection",
        "unit": "",
        "icon": "mdi:wifi",
        "datafield": "connection_status",
    },
    "date_time": {
        "type": "Datetime",
        "unit": "",
        "icon": "mdi:calendar-clock",
        "datafield": "date_time",
    },
    CH_CURRENT_TEMPERATURE: {
        "type": "Current Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "room_temp",
    },
    "outside_temp": {
        "type": "Outside Temp",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "outside_temp",
    },
    "outside_temp_avg": {
        "type": "Average Outside Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "tout_avg",
    },
    WEATHER_STATUS: {
        "type": "Weather Status",
        "unit": "",
        "icon": "mdi:white-balance-sunny",
        "datafield": "weather_status",
    },
    "pcb_temp": {
        "type": "PCB Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "pcb_temp",
    },
    CH_TEMPERATURE: {
        "type": "Target Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "shown_set_temp",
    },
    CH_OPERATION_MODE: {
        "type": "Operation Mode",
        "unit": "",
        "icon": "mdi:settings",
        "datafield": "ch_mode",
    },
    "ch_water_pressure": {
        "type": "Central Heating Pressure",
        "unit": "bar",
        "icon": "mdi:gauge",
        "datafield": "ch_water_pres",
    },
    "ch_water_temp": {
        "type": "CH Water Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "ch_water_temp",
    },
    "ch_return_temp": {
        "type": "CH Return Temperature",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "ch_return_temp",
    },
    DHW_CURRENT_TEMPERATURE: {
        "type": "Hot Water Temp",
        "unit": "°C",
        "icon": "mdi:thermometer",
        "datafield": "dhw_water_temp",
    },
    "dhw_water_pres": {
        "type": "Hot Water Pressure",
        "unit": "bar",
        "icon": "mdi:gauge",
        "datafield": "dhw_water_pres",
    },
    "dhw_flow_rate": {
        "type": "Hot Water Flow Rate",
        "unit": "?",
        "icon": "mdi:gauge",
        "datafield": "dhw_flow_rate",
    },
    BOILER_STATUS: {
        "type": "Boiler Status",
        "unit": "",
        "icon": "mdi:flash",
        "datafield": "boiler_status",
    },
    BOILER_CONF: {
        "type": "Boiler Config",
        "unit": "",
        "icon": "mdi:flash",
        "datafield": "boiler_config",
    },
    "burning_hours": {
        "type": "Burning Hours",
        "unit": "h",
        "icon": "mdi:fire",
        "datafield": "burning_hours",
    },
    "voltage": {
        "type": "Voltage",
        "unit": "V",
        "icon": "mdi:flash",
        "datafield": "voltage",
    },
    "current": {
        "type": "Current",
        "unit": "mA",
        "icon": "mdi:flash-auto",
        "datafield": "current",
    },
    "flame_level": {
        "type": "Flame",
        "unit": "%",
        "icon": "mdi:fire",
        "datafield": "rel_mod_level",
    },
    REPORT_TIME: {
        "type": "Report Time",
        "unit": "",
        "icon": "mdi:clock",
        "datafield": "report_time",
    },
}


for i in CONTROLS:
    if i not in SENSOR_TYPES:
        SENSOR_TYPES[i] = [i, "", "mdi:flash", CONTROLS[i]]

REPORT_STRUCTURE_INV = {v: i for i in REPORT_STRUCTURE for v in REPORT_STRUCTURE[i]}

BOILER_STATES = {
    14: "Heating CV & Water",
    12: "Heating Water",
    10: "Heating CV",
    8: "Heating Boiler",
    4: "Pumping Water",
    2: "Pumping CV",
    0: "Idle",
}

CONNECTION_STATES = {23: {"state": "Connected to BCU"}}

WEATHER_STATES = {
    0: {"state": "Sunny", "icon": "mdi:weather-sunny"},
    1: {"state": "Clear", "icon": "mdi:weather-night"},
    2: {"state": "Rainy", "icon": "mdi:weather-rainy"},
    # Not sure, Atag icons unclear
    3: {"state": "Snowy", "icon": "mdi:weather-snowy"},
    # Not sure, Atag icons unclear
    4: {"state": "Haily", "icon": "mdi:weather-hail"},
    5: {"state": "Windy", "icon": "mdi:weather-windy"},
    6: {"state": "Misty", "icon": "mdi:weather-fog"},
    7: {"state": "Cloudy", "icon": "mdi:weather-cloudy"},
    8: {"state": "Partly Sunny", "icon": "mdi:weather-partlycloudy"},
    9: {"state": "Partly Cloudy", "icon": "mdi:cloud"},  # Night with clouds..
    # Not sure, Atag icons unclear
    10: {"state": "Shower", "icon": "mdi:weather-pouring"},
    11: {"state": "Lightning", "icon": "mdi:weather-lightning"},
    12: {"state": "Hurricane", "icon": "mdi:weather-hurricane"},
    13: {"state": "Unknown", "icon": "mdi:cloud-question"},
}

MODES = {
    STATE_MANUAL: 1,
    STATE_AUTO: 2,
    STATE_HOLIDAY: 3,
    STATE_EXTEND: 4,
    STATE_FIREPLACE: 5,
}
INT_MODES = {v: {"state": k} for k, v in MODES.items()}

# Non weather based and weather based respectively
CH_CONTROLS = {"heat": 0, "auto": 1}
INT_CH_CONTROLS = {v: {"state": k} for k, v in CH_CONTROLS.items()}

SENSOR_VALUES = {
    "report_time": "time",
    "date_time": "time",
    "weather_status": WEATHER_STATES,
    "connection_status": CONNECTION_STATES,
    "boiler_status": BOILER_STATES,
    "ch_control_mode": INT_CH_CONTROLS,
    "ch_mode": INT_MODES,
    "boiler_config": "int",
    "ch_status": "int",
    "dhw_status": "int",
    "device_status": "int",
}

SQLTYPES = {
    float: "FLOAT",
    int: "INT",
    datetime.datetime: "DATETIME",
    str: "TEXT",
    list: "TEXT",
    dict: "TEXT",
    type(None): "TEXT",
}




#-*- coding:utf-8 -*-
'''
Provides connection to ATAG One Thermostat REST API
'''
from .gateway import AtagDataStore
from .errors import AtagException
from .const import *
from .discovery import discover_atag